### R code from vignette source 'robustfa.Rnw'

###################################################
### code chunk number 1: robustfa.Rnw:176-178
###################################################
## set the prompt to "R> " and the continuation to "+ "
options(prompt = "R> ", continue = "+  ", width = 70, useFancyQuotes = FALSE)


###################################################
### code chunk number 2: intro
###################################################
##
## Load the 'robustfa' package and two data sets
##
library("robustfa")
data("hbk")
hbk.x <- hbk[,1:3]              # take only the X part

data("stock611")
stock608 = stock611[-c(92,2,337),]
stock604 = stock611[-c(92,2,337,338,379,539,79),]
R611 = cor(stock611[,3:12])


###################################################
### code chunk number 3: FaClassic-factorScore
###################################################
faClassic = FaClassic(x = hbk.x, factors = 2, method = "pca",
scoresMethod = "regression"); faClassic
summary(faClassic)
plot(faClassic, which = "factorScore", choices = 1:2)



###################################################
### code chunk number 4: FaCov-factorScore
###################################################
faCov = FaCov(x = hbk.x, factors = 2, method = "pca",
scoresMethod = "regression", cov.control = CovControlMcd()); faCov
summary(faCov)
plot(faCov, which = "factorScore", choices = 1:2)



###################################################
### code chunk number 5: fa-ex-hbk
###################################################
##
## Plot of the first two factors of hbk: classical and robust.
##
usr <- par(mfrow = c(1,2))
cfaClassic <- list(center = c(0,0), cov = diag(faClassic@eigenvalues[1:2]), n.obs = faClassic@n.obs)
rrcov:::.myellipse(faClassic@scores, xcov = cfaClassic, main = "Classical",  xlab = "Factor1", ylab = "Factor2", id.n = 0)
abline(v = 0)
abline(h = 0)
text(30,-20,labels = "1-13", cex = 0.8)
text(60,-33,labels = "14", cex = 0.8)

cfaCov <- list(center = c(0,0), cov = diag(faCov@eigenvalues[1:2]), n.obs = faCov@n.obs)
rrcov:::.myellipse(faCov@scores, xcov = cfaCov, main = "Robust (MCD)",
xlab = "Factor1", ylab = "Factor2", xlim = c(-10,45), ylim = c(-5,20), id.n = 4)
# xlim = c(-10,45), ylim = c(-25,15),
text(30,10,labels = "1-10", cex = 0.8)
abline(v = 0)
abline(h = 0)
par(usr)

colMeans(faClassic@scores)
colMeans(faCov@scores)
colMeans(faClassic@scores[15:75,])
colMeans(faCov@scores[15:75,])


###################################################
### code chunk number 6: show-print-summary_Fa
###################################################
##
## facovRegOgk is obtained from FaCov.default,
##               uses the default method = "mle"
##
facovRegOgk = FaCov(x = scale(stock611[,3:12]), factors = 3,
cov.control = CovControlOgk(), scoresMethod = "regression")


###################################################
### code chunk number 7: show-print-summary_Fa
###################################################
facovRegOgk
summary(facovRegOgk)


###################################################
### code chunk number 8: show-print-summary_Fa
###################################################
## In fact, it is equivalent to use FaCov.formula
## facovForRegOgkMle = facovRegOgk
facovForRegOgkMle = FaCov(~., data = as.data.frame(scale(stock611[,3:12])),
factors = 3, cov.control = CovControlOgk(), method = "mle",
scoresMethod = "regression")


###################################################
### code chunk number 9: show-print-summary_Fa
###################################################
facovForRegOgkMle
summary(facovForRegOgkMle)


###################################################
### code chunk number 10: show-print-summary_Fa
###################################################
## obj = show(obj) = print(obj) = myFaPrint(obj)
class(facovRegOgk)


###################################################
### code chunk number 11: show-print-summary_Fa
###################################################
show(facovRegOgk)


###################################################
### code chunk number 12: show-print-summary_Fa
###################################################
facovRegOgk
print(facovRegOgk)
myFaPrint(facovRegOgk)


###################################################
### code chunk number 13: show-print-summary_Fa
###################################################
summaryFacovRegOgk = summary(facovRegOgk); summaryFacovRegOgk


###################################################
### code chunk number 14: show-print-summary_Fa
###################################################
show(summaryFacovRegOgk)
class(summaryFacovRegOgk)


###################################################
### code chunk number 15: predict_Fa
###################################################
## If missing newdata, the scores are used
predict(facovRegOgk)


###################################################
### code chunk number 16: predict_Fa
###################################################
predict(facovRegOgk)[c(1:5, 607:611),]


###################################################
### code chunk number 17: predict_Fa
###################################################
## If not missing newdata, newdata must have the same center and
## scale attributes as the original data.
newdata = stock611[1,3:12]
x = scale(stock611[,3:12])
newdata = scale(newdata, center = attr(x,"scaled:center"),
                          scale = attr(x,"scaled:scale"))


###################################################
### code chunk number 18: predict_Fa
###################################################
## To compute prediction, newdata should be scaled using the robust center,
## this is done in "FaCov.default" when computing the scores:
## scores <- computeScores(out,
##           newdata = scale(data, center = covmat$center, scale = F),
##           scoresMethod = scoresMethod)
## Now, prediction = predict(facovRegOgk)[1,] = facovRegOgk@scores[1,]
newdata = scale(newdata, center = facovRegOgk@center, scale = F)
prediction = predict(facovRegOgk, newdata = newdata)
prediction


###################################################
### code chunk number 19: plot-Fa-FaClassic-FaCov
###################################################
## method = "mle" error
##  faclassicReg611 = FaClassic(x = scale(stock611[,3:12]), factors = 3,
##        scoresMethod = "regression"); faclassicReg611
## Error in factanal(factors = factors, covmat = covmat) :
##   unable to optimize from these starting value(s)

## method = "pca" OK
## x = scale(stock611[,3:12])

## FaClassic
faclassicRegPca611 = FaClassic(x = scale(stock611[,3:12]), factors = 2,
method = "pca", scoresMethod = "regression"); faclassicRegPca611
summary(faclassicRegPca611)


###################################################
### code chunk number 20: plot-Fa-FaClassic-FaCov
###################################################
## FaCov
facovRegOgkPca = FaCov(x = scale(stock611[,3:12]), factors = 2, method = "pca",
cov.control = CovControlOgk(), scoresMethod = "regression"); facovRegOgkPca
summary(facovRegOgkPca)


###################################################
### code chunk number 21: plot-Fa-FS_reg_c_ogk
###################################################
usr <- par(mfrow=c(1,2))
plot(faclassicRegPca611, which = "factorScore", choices = 2:1)
plot(facovRegOgkPca, which = "factorScore", choices = 1:2)
par(usr)


###################################################
### code chunk number 22: plot-Fa-scree_reg_c_ogk
###################################################
usr <- par(mfrow=c(1,2))
plot(faclassicRegPca611, which = "screeplot")
plot(facovRegOgkPca, which = "screeplot")
par(usr)


###################################################
### code chunk number 23: plot-Fa-FS_ellipse_reg_c_ogk
###################################################
## Plot of the first two factors of stock611 and 97.5% tolerance ellipse plot:
## classical and robust.
cfaclassicRegPca611 <- list(center = c(0,0), cov = diag(faclassicRegPca611@eigenvalues[1:2]), n.obs = faclassicRegPca611@n.obs)
cfacovRegOgkPca <- list(center = c(0,0), cov = diag(facovRegOgkPca@eigenvalues[1:2]), n.obs = facovRegOgkPca@n.obs)

usr <- par(mfrow=c(1,2))
rrcov:::.myellipse(faclassicRegPca611@scores[,2:1], xcov = cfaclassicRegPca611,
main = "Classical, method = \"pca\"", xlab = "Factor2", ylab = "Factor1", id.n = 0, xlim = c(-6,6), ylim = c(-5,30))
abline(v = 0)
abline(h = 0)
rrcov:::.myellipse(facovRegOgkPca@scores[,1:2], xcov = cfacovRegOgkPca,
main = "Robust (OGK), method = \"pca\"", xlab = "Factor1", ylab = "Factor2", id.n = 0, xlim = c(-6,6), ylim = c(-5,30))
# xlim = c(-6,6), ylim = c(-5,30)
# text(30,10,labels = "1-10", cex = 0.8)
abline(v = 0)
abline(h = 0)
par(usr)


###################################################
### code chunk number 24: plot-Fa-colMeans-orderedFs
###################################################
##
## the colMeans of all observations of FaClassic-class equals 0
## the colMeans of good observations (excluding the outliers) of FaCov-class equals 0
colMeans(faclassicRegPca611@scores)
colMeans(facovRegOgkPca@scores)

## Display scores and ordered scores.
faclassicRegPca611@scores[,2:1]
facovRegOgkPca@scores[,1:2]


###################################################
### code chunk number 25: plot-Fa-colMeans-orderedFs
###################################################
orderedFsC = fsOrder(faclassicRegPca611@scores[,2:1]); orderedFsC


###################################################
### code chunk number 26: plot-Fa-colMeans-orderedFs
###################################################
Lst=list(orderedFsC[[1]][1:10,], orderedFsC[[2]][1:10,]); Lst


###################################################
### code chunk number 27: plot-Fa-colMeans-orderedFs
###################################################
orderedFsOgk = fsOrder(facovRegOgkPca@scores[,1:2]); orderedFsOgk


###################################################
### code chunk number 28: plot-Fa-colMeans-orderedFs
###################################################
Lst=list(orderedFsOgk[[1]][1:10,], orderedFsOgk[[2]][1:10,]); Lst


###################################################
### code chunk number 29: plot_Cov-compute
###################################################
##
## all the following plots are OK for x = covOgk
## myplotDD shows id.n and ind
##
covOgk = CovRobust(x = scale(stock611[,3:12]), control = "ogk"); covOgk
covOgk68 = CovRobust(x = scale(stock611[,c(8,10)]), control = "ogk")


###################################################
### code chunk number 30: plot_Cov-cov_ogk_dd
###################################################
plot(x = covOgk, which = "dd")


###################################################
### code chunk number 31: plot_Cov-cov_ogk_dd_my
###################################################
result = myplotDD(x = covOgk)


###################################################
### code chunk number 32: plot_Cov-cov_ogk_distance_classical
###################################################
plot(x = covOgk, which = "distance", classic = T)


###################################################
### code chunk number 33: plot_Cov-cov_ogk_qqchi2_classical
###################################################
plot(x = covOgk, which = "qqchi2", classic = T)


###################################################
### code chunk number 34: plot_Cov-cov_ogk_tolEllipsePlot_classical_6_8
###################################################
plot(x = covOgk68, which = "tolEllipsePlot", classic = T)


###################################################
### code chunk number 35: plot_Cov-cov_ogk_tolEllipsePlot_classical_6_8
###################################################
## plot(x = covOgk, which = "tolEllipsePlot", classic = T)
## Warning message:
## In .local(x, y, ...) :
##   Warning: For tolerance ellipses the dimension must be 2!


###################################################
### code chunk number 36: plot_Cov-cov_ogk_screeplot_classical
###################################################
plot(x = covOgk, which = "screeplot", classic = T)


###################################################
### code chunk number 37: plot_Cov-CovClassic
###################################################
##
## can not plot "tolEllipsePlot"?
## For tolerance ellipses the dimension must be 2!
##
## covC = CovClassic(x = scale(stock611[,3:12])); covC # covC = R611
## plot(x = covC, which = "all")

## Tolerance ellipses plot for variables x6 and x8.
## It is the same with
## plot(x = covOgk68, which = "tolEllipsePlot", classic = T)
covC68Scale = CovClassic(x = scale(stock611[,c(8,10)]))
plot(x = covC68Scale, which = "tolEllipsePlot")


###################################################
### code chunk number 38: cutoff-id.n-sort.y-ind
###################################################
cat("cutoff =", result$cutoff, "\n")
cat("id.n <- length(which(rd>cutoff))\n")
cat("id.n =", result$id.n, "\n")
cat("Here y is the robust distance (rd).\n")

Lst = list(x = result$sort.y$x[c(1:5, 607:611)], ix = result$sort.y$ix[c(1:5, 607:611)])
cat("sort.y = (To save space, only the smallest five and largest five
elements of sort.y$x and sort.y$ix are shown.)\n"); show(Lst)
cat("ind =\n"); show(result$ind)


###################################################
### code chunk number 39: get_Fa
###################################################
##
## accessor functions
##
getEigenvalues(facovRegOgk)


###################################################
### code chunk number 40: get_Fa
###################################################
getCenter(facovRegOgk)
getFa(facovRegOgk)
getLoadings(facovRegOgk)
getQuan(facovRegOgk)
getScores(facovRegOgk)
getSdev(facovRegOgk)


